<?php
namespace Teknisa\SM\Factory;

use Doctrine\ORM\EntityManager;

class Sm00004000ItemContratoFactory
{
    protected $connection;
    
    public function __construct(EntityManager $entityManager) {
        
        $this->connection = $entityManager->getConnection();
    }
    /**
     *Valida se o percentual máximo de desconto dado pelo o vendedor esta conforme com seu perfil de acesso ou operador
     */
    public function validaPorcentualDescontoAcrescimoSave(array $params) {
        $sql = "SELECT NVL(VRPERCENTUALOPER, VRPERCENTUALPERF) AS VRPERCDESC, CDSERVORCA, TIPO
                FROM(
                    SELECT MAX(VRPERCENTUAL) AS VRPERCENTUALPERF, NULL AS VRPERCENTUALOPER, CDSERVORCA, 2 AS TIPO
                    FROM FCT_PERCDESCPERFIL
                    WHERE NRPERFILACESSO  IN (SELECT NRPERFILACESSO 
                                                FROM PERFACESSPARC 
                                               WHERE NRPARCNEGOCIO IN (SELECT NRPARCNEGOCIO 
                                                                          FROM OPERADOR 
                                                                          WHERE CDOPERADOR = :CDOPERADOR))
                    AND (CDSERVORCA = :CDSERVORCA
                     OR CDSERVORCA IS NULL)
                    AND IDTIPO = :IDTIPO
                    AND NRORG = :NRORG
                GROUP BY CDSERVORCA, NULL, 2
                    
                    UNION ALL 
                    
                    SELECT NULL AS VRPERCENTUALPERF, VRPERCDESC AS VRPERCENTUALOPER, CDSERVORCA, 1 AS TIPO
                    FROM FCT_PERCDESCOPER
                    WHERE NRPARCNEGOCIO IN (SELECT NRPARCNEGOCIO 
                                              FROM OPERADOR 
                                              WHERE CDOPERADOR = :CDOPERADOR)
                    AND (CDSERVORCA = :CDSERVORCA
                     OR CDSERVORCA IS NULL)
                    AND IDTIPO = :IDTIPO
                    AND NRORG = :NRORG
                    )
          ORDER BY TIPO, CDSERVORCA, VRPERCDESC DESC";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
        
    /**
     * Deleta da tabela de agendamentos pendentes os items deletados no 
     * contrato aprovado
    **/
    public function excluiItemAgendPendente(array $params){
        $sql = "DELETE FROM FCT_AGENDPENDENTE
                WHERE NRSEQCONTRATO     = :NRSEQCONTRATO
                    AND NRITEMOCONTRATO = :NRITEMOCONTRATO
                    AND NRORG           = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
        
    /**
     * Deleta da tabela de agendamentos pendentes os items deletados no 
     * contrato aprovado
    **/
    public function alteraValorQuantidadeAgendPendente(array $params){
        $sql = "UPDATE FCT_AGENDPENDENTE
                SET QTPENDENCIA = :QTPENDENCIA
                WHERE NRSEQCONTRATO     = :NRSEQCONTRATO
                    AND NRITEMOCONTRATO = :NRITEMOCONTRATO
                    AND NRORG           = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     * retorna a quantidade de itens que já estao agendados
    **/
    public function getQuantidadeItensAgendados(array $params){
        $sql = "SELECT COUNT(*) AS QTITENSAGEND
                FROM FCT_RECURSOSAGENDCONT AGENDREC
                INNER JOIN FCT_AGENDRECURSOCONT AGEND
                    ON AGENDREC.NRAGENDRECURSOCONT = AGEND.NRAGENDRECURSOCONT
                    AND AGENDREC.NRORG              = AGEND.NRORG
                WHERE AGEND.NRSEQCONTRATO     = :NRSEQCONTRATO
                    AND AGENDREC.NRITEMOCONTRATO = :NRITEMOCONTRATO
                    AND AGENDREC.NRORG           = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * retorna a quantidade de itens que já estao agendados
    **/
    public function getItemAgendamentoPendente(array $params){
        $sql = "SELECT COUNT(*) AS QUANTIDADEITEM
                FROM FCT_AGENDPENDENTE
                WHERE NRSEQCONTRATO     = :NRSEQCONTRATO
                    AND NRITEMOCONTRATO = :NRITEMOCONTRATO
                    AND NRORG           = :NRORG ";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Deleta as formções alteradas e não utilizadas
    **/
    public function deletaAlteracoesInutiFormCont(array $params) {
        $sql = "DELETE FROM FCT_CALCFORMPRECSERV_ITCONT
                    WHERE (NRSEQCONTRATO    = :NRSEQCONTRATO
                        AND NRORG           = :NRORG
                        AND NRITEMOCONTRATO = :NRITEMOCONTRATO)
                        OR (:NRITEMOCONTRATO IS NULL AND NRITEMOCONTRATO IS NULL)";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     * Insere o numero do item gerado na formação de preço auxiliar
    **/
    public function editaItensFormacaoAuxCont(array $params, $iditemcalculo) {
        $sql = "UPDATE FCT_CALCFORMPRECSERV_ITCONT
                SET NRITEMOCONTRATO = :NRITEMOCONTRATO
                WHERE NRSEQCONTRATO          = :NRSEQCONTRATO
                      AND NRFORMPRECOSERVICO = :NRFORMPRECOSERVICO
                      AND NRORG              = :NRORG
                      AND IDITEMCALCULO IN('".$iditemcalculo."')
                      AND NRITEMOCONTRATO IS NULL";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     * Deleta o item de formação de preço inserido quando a rotina de alteração do número 
     * do item na tabela de formção de preço auxiliar não funciona
    **/
    public function deletarItensFormacao(array $params, $nritemocontrato){
        $sql = "DELETE FROM FCT_ITEMCONTRATO
                WHERE NRSEQCONTRATO     = :NRSEQCONTRATO
                    AND NRITEMOCONTRATO IN(".$nritemocontrato.")
                    AND NRORG           = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }

    /**
     * deleta a árvore completa de elementos alterados na formação de 
     * preço auxiliar já inseridA no contrato
     **/
    public function deletarArvoreFormPrecoItCont(array $params){
        $sql = "DELETE FROM FCT_CALCFORMPRECSERV_ITCONT
                WHERE NRCALCFORMPRECSERV IN(SELECT NRCALCFORMPRECSERV
                                                FROM FCT_CALCFORMPRECSERV_ITCONT
                                                WHERE NRITEMOCONTRATO      = :NRITEMOCONTRATO
                                                    AND NRSEQCONTRATO      = :NRSEQCONTRATO
                                                    AND NRFORMPRECOSERVICO = :NRFORMPRECOSERVICO
                                                    AND NRORG              = :NRORG
                                                CONNECT BY PRIOR NRCALCFORMPRECSERV = NRCALCFORMPRECSERVPAI
                                                START WITH NRCALCFORMPRECSERV = :NRCALCFORMPRECSERV)";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    /**
     * manda alterar o valor do item na proposta
    **/
    public function salvarItemComposicaoProposta(array $params){
        $sql = "UPDATE FCT_ITEMORCAMENTO
                SET QTMEDIDAORCA     = ROUND(:QTMEDIDAORCA, 2),
                    VRUNITITEMORCA   = ROUND(:VRUNITITEMORCA, 2),
                    VRTOTALITEMORCA  = ROUND(:VRTOTALITEMORCA, 2),
                    VRITEMSEMDESC    = ROUND(:VRITEMSEMDESC, 2)
                WHERE NRSEQORCAMENTO    = :NRSEQORCAMENTO
                    AND NRITEMORCAMENTO = :NRITEMORCAMENTO
                    AND CDSERVORCA      = :CDSERVORCA";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    /**
     * Busca os itens na proposta
    **/
    public function getItensProposta(array $params){
        $sql = "SELECT CDSERVORCA,
                    VRUNITITEMORCA,
                    VRTOTALITEMORCA,
                    QTITEMORCAMENTO
                FROM FCT_ITEMORCAMENTO
                WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                    AND NRORG = :NRORG
                    AND IDELEMENTO IS NULL";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    /**
     * Atualiza no contrato o valor do item
    **/
    public function atualizarItemContComposicao(array $params){
        $sql = "UPDATE FCT_ITEMCONTRATO
                SET VRUNITITEMCONT  = ROUND(:VRUNITITEMCONT, 2),
                    VRTOTITEMCONT   = ROUND(:VRTOTALITEMCONT, 2),
                    QTITEMCONTRATO  = ROUND(:QTITEMCONTRATO, 2),
                    CDOPERINCLUSAO  = :CDOPERADOR
                WHERE NRSEQCONTRATO = :NRSEQCONTRATO
                    AND NRORG       = :NRORG
                    AND CDSERVORCA  = :CDSERVORCA";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }

    public function verificarSeExisteDescontoNegociacaoContrato(array $params) {
        $sql = "SELECT NVL(VRPERCDESCITEM, 0) AS VRPERCDESCITEM
                  FROM FCT_NEGOCIACAOCONTRATO
                 WHERE NRNEGOCIACAOCONTRATO = :NRNEGOCIACAOCONTRATO
                   AND NRORG                = :NRORG";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }

    public function getSomaPercentualDescontoItensNegociacaoContrato(array $params) {
        $sql = "SELECT NVL(SUM(VRPERCDESCITEM), 0) AS VRPERCDESCITEM
                  FROM FCT_ITNEGOCIACAOCONTRATO
                 WHERE NRNEGOCIACAOCONTRATO = :NRNEGOCIACAOCONTRATO
                   AND NRORG                = :NRORG";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }

    public function getServicosNegociacaoContrato(array $params) {
        $sql = "SELECT DISTINCT CDSERVORCA
                  FROM FCT_ITNEGOCIACAOCONTRATO
                 WHERE NRNEGOCIACAOCONTRATO = :NRNEGOCIACAOCONTRATO
                   AND NRORG                = :NRORG";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }

    public function getCodigoNegociacaoDoItem(array $params) {
        $sql = "SELECT NRNEGOCIACAOCONTRATO
                  FROM FCT_ITNEGOCIACAOCONTRATO
                 WHERE NRITEMOCONTRATO = :NRITEMOCONTRATO
                   AND NRORG           = :NRORG";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }

    public function atualizarValorItemNegociacao(array $params) {
        $sql = "UPDATE FCT_ITNEGOCIACAOCONTRATO
                   SET VRTOTITEM        = ROUND(:VRTOTITEM, 2),
                       VRDESCITEM       = ROUND(:VRTOTITEM * (NVL(VRPERCDESCITEM, 0) / 100), 2),
                       VRTOTITEMCOMDESC = ROUND(:VRTOTITEM - (:VRTOTITEM * (NVL(VRPERCDESCITEM, 0) / 100)), 2),
                       DTULTATU         = SYSDATE,
                       CDOPERULTATU     = :CDOPERULTATU,
                       NRORGULTATU      = :NRORGULTATU
                 WHERE NRNEGOCIACAOCONTRATO = :NRNEGOCIACAOCONTRATO
                   AND NRITEMOCONTRATO      = :NRITEMOCONTRATO
                   AND NRORG                = :NRORG";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }

    public function getNegociacoesContrato(array $params) {
        $sql = "SELECT NRNEGOCIACAOCONTRATO, NRSEQCONTRATO, NRTIPOORCAMENTO, IDTPFATURA, CDFILIALFATURA, 
                       NRPARCFATURA, CDTIPOOPERFATURA, CDSERIENF, CDEMPRESA, CDINSCESTA,
                       CDINSCMUNI, VRTOTITEM, VRPERCDESCITEM, VRDESCITEM, VRTOTITEMCOMDESC, 
                       NRNEGOCIACAOPROPOSTA, IDATIVO, NRORG, IDTPDESCONTO, NRESTRUTURAM, 
                       IDPREVCOBRFUT, CDPAISFAT, SGESTADOFAT, CDMUNICIPIOFAT, CDTRIBPRODPADR, 
                       IDNATUOPER
                  FROM FCT_NEGOCIACAOCONTRATO
                 WHERE NRSEQCONTRATO = :NRSEQCONTRATO
                   AND NRORG         = :NRORG";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }

    public function getSomaValorItensNegociacao(array $params) {
        $sql = "SELECT SUM(VRTOTITEMCOMDESC) AS VRTOTAL
                  FROM FCT_ITNEGOCIACAOCONTRATO
                 WHERE NRNEGOCIACAOCONTRATO = :NRNEGOCIACAOCONTRATO
                   AND NRORG                = :NRORG
                   AND NRITEMCONTRATOPAI   IS NULL";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }

    public function atualizarValorNegociacaoComSomaDosItens(array $params) {
        $sql = "UPDATE FCT_NEGOCIACAOCONTRATO
                   SET VRTOTITEM        = ROUND(:VRTOTAL, 2),
                       VRDESCITEM       = ROUND((:VRTOTAL * NVL(VRPERCDESCITEM, 0) / 100), 2),
                       VRTOTITEMCOMDESC = ROUND(:VRTOTAL - (:VRTOTAL * NVL(VRPERCDESCITEM, 0) / 100), 2),
                       DTULTATU         = SYSDATE,
                       CDOPERULTATU     = :CDOPERULTATU,
                       NRORGULTATU      = :NRORGULTATU
                 WHERE NRNEGOCIACAOCONTRATO = :NRNEGOCIACAOCONTRATO
                   AND NRORG                = :NRORG";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }
    /**
     * Retorna as informações de nome contratual da proposta ligada a um contrato
    **/
    public function getInfoNomeContratualOrca(array $params) {
        $sql = "SELECT DISTINCT ITEM.NMITEMORCAMENTO,
                  NVL(ITEM.NMCONTRATUALITEMORCA, ITEM.NMITEMORCAMENTO) AS NMCONTRATUALITEMORCA,
                  ITEM.IDELEMENTO,
                  ITEM.NRORG,
                  ITEM.NRSEQORCAMENTO
                FROM FCT_ITEMORCAMENTO ITEM
                JOIN FCT_REFEORCACONT REFE
                  ON ITEM.NRSEQORCAMENTO = REFE.NRSEQORCAMENTO
                  AND ITEM.NRORG         = REFE.NRORG
                WHERE ITEM.IDELEMENTO IS NOT NULL
                  AND REFE.NRSEQCONTRATO = :NRSEQCONTRATO
                  AND ITEM.NRORG         = :NRORG
                  AND REFE.IDATIVO       = 'S'";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    /**
     * Salva as alterações realizadas no nome contratual dos itens 
    **/
    public function salvarNomeContratualContrato(array $params) {
        $sql = "UPDATE FCT_ITEMCONTRATO
                SET NMCONTRATUALITEMCONT = :NMCONTRATUALITEMCONT,
                    CDOPERULTATU         = :CDOPERADOR
                WHERE NRSEQCONTRATO  = :NRSEQCONTRATO
                    AND IDELEMENTO   = :IDELEMENTO
                    AND NRORG        = :NRORG";
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }
    /**
     * Retorna o somatório dos itens de determinado servico
    **/
    public function getValorTotalServico(array $params) {
        $sql = "SELECT SUM(VRTOTITEMCONT) AS VRTOTALSERVORCA
                FROM FCT_ITEMCONTRATO
                WHERE NRSEQCONTRATO = :NRSEQCONTRATO
                    AND CDSERVORCA  = :CDSERVORCA
                    AND NRORG       = :NRORG
                    AND IDELEMENTO IS NOT NULL";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll()[0]["VRTOTALSERVORCA"];
    }
    /**
     * Retorna o item pai de um determinado serviço do contrato
    **/
    public function getItemPaiArvore(array $params) {
        $sql = "SELECT NRSEQCONTRATO,
                    NRITEMOCONTRATO,
                    QTMEDIDACONTR,
                    VRUNITITEMCONT,
                    VRTOTITEMCONT,
                    CDSERVORCA
                FROM FCT_ITEMCONTRATO
                WHERE NRSEQCONTRATO = :NRSEQCONTRATO
                    AND CDSERVORCA  = :CDSERVORCA
                    AND NRORG       = :NRORG
                    AND IDELEMENTO IS NULL";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    /**
     * altera o valor total de um item do contrato
    **/
    public function replicarAlteracoesParaItensPais(array $params) {
        $sql = "UPDATE FCT_ITEMCONTRATO
                SET NRSEQCONTRATO   = :NRSEQCONTRATO,
                    VRUNITITEMCONT  = ROUND(:VRUNITITEMCONT, 2),
                    VRTOTITEMCONT   = ROUND(:VRTOTITEMCONT, 2),
                    CDOPERULTATU    = :CDOPERULTATU
                WHERE NRSEQCONTRATO     = :NRSEQCONTRATO
                    AND CDSERVORCA      = :CDSERVORCA
                    AND NRORG           = :NRORG
                    AND NRITEMOCONTRATO = :NRITEMOCONTRATO
                    AND IDELEMENTO IS NULL";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    /**
     * retorna os itens de uma proposta gerada pelo contrato
    **/
    public function getItensPropostaGeradaPorContrato(array $params) {
        $sql = "SELECT ITEM.NRSEQORCAMENTO,
                  ITEM.NRITEMORCAMENTO,
                  ROUND(ITEM.QTMEDIDAORCA, 2) AS QTMEDIDAORCA,
                  ROUND(ITEM.VRUNITITEMORCA, 2) AS VRUNITITEMORCA,
                  ROUND(ITEM.VRPERCDESCORCA, 2) AS VRPERCDESCORCA,
                  ROUND(ITEM.VRDESCORCA, 2) AS VRDESCORCA,
                  ITEM.IDTPDESCORCA,
                  ROUND(ITEM.VRTOTALITEMORCA, 2) AS VRTOTALITEMORCA,
                  ROUND(ITEM.VRITEMSEMDESC, 2) AS VRITEMSEMDESC,
                  ITEM.CDSERVORCA,
                  ITEM.CDTPMEDIDAVARIAVEL,
                  ITEM.NRTIPOORCAMENTO
                FROM FCT_ITEMORCAMENTO ITEM
                JOIN FCT_REFEORCACONT REFE
                  ON ITEM.NRSEQORCAMENTO = REFE.NRSEQORCAMENTO
                  AND ITEM.NRORG         = REFE.NRORG
                JOIN FCT_CONTRATO CONT
                    ON REFE.NRSEQCONTRATO = CONT.NRSEQCONTRATO
                    AND REFE.NRORG        = CONT.NRORG
                WHERE REFE.IDGERADOPOR        = 'CONTRATO'
                  AND REFE.NRSEQCONTRATO      = :NRSEQCONTRATO
                  AND REFE.NRORG              = :NRORG
                  AND ITEM.IDELEMENTO         = :IDELEMENTO 
                  AND ITEM.CDSERVORCA         = :CDSERVORCA
                  AND ITEM.CDTPMEDIDAVARIAVEL = :CDTPMEDIDAVARIAVEL
                  AND ITEM.CDTPMEDIDASERV     = :CDTPMEDIDASERV
                  AND CONT.NRVERSAOCONTRATO   = 1";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    } 
    /**
     * Retorna as variações de preço do serviço 
    **/
    public function getVariacoesPrecoDoServico(array $params, $cdservorca) {
        $sql = "SELECT DISTINCT NVL(QTMEDIDAINIVARIACAO, 1) AS QTMEDIDAINIVARIACAO,
                       NVL(QTMEDIDAFIMVARIACAO, 99999) AS QTMEDIDAFIMVARIACAO,
                       NVL(VRPRECOVARIACAO, ITVAR.VRPRECO) AS VRPRECOVARIACAO,
                       NVL(IDMULTIPVARIACAO, 'N') AS IDMULTIPVARIACAO
                FROM FCT_ITPRECBASESERV ITVAR
                LEFT JOIN FCT_VARIACAOPRECBASESERV VAR
                  ON VAR.NRSEQITPRECBSERV  = ITVAR.NRSEQITPRECBSERV
                  AND VAR.NRORG            = ITVAR.NRORG
                WHERE ITVAR.CDSERVORCA IN($cdservorca)
                    AND ITVAR.NRORG = :NRORG
                    AND ITVAR.CDTPMEDIDASERV = :CDTPMEDIDASERV
                    AND (VAR.CDTPMEDIDAVARIAVEL IS NULL
                         OR VAR.CDTPMEDIDAVARIAVEL = :CDTPMEDIDAVARIAVEL)";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    public function getSomaValorElementosItemNegociacao(array $params) {
        $sql = "SELECT SUM(VRTOTITEMCOMDESC) AS VRTOTAL
                  FROM FCT_ITNEGOCIACAOCONTRATO
                 WHERE NRNEGOCIACAOCONTRATO = :NRNEGOCIACAOCONTRATO
                   AND NRITEMCONTRATOPAI    = :NRITEMCONTRATOPAI
                   AND NRORG                = :NRORG";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }

    public function getParametrosFaturamentoParaItemNegociacaoContrato($params) {
        $sql = "SELECT PARAM.NRTIPOORCAMENTO,
                       PARAM.NRITEMOCONTRATO,
                       PARAM.NRITEMCONTRATOPAI,
                       PARAM.CDSERVORCA,
                       PARAM.VRTOTITEMCONT,
                       PARAM.NMITEMCONTRATO,
                       PARAM.IDTPFATURA,
                       PARAM.CDTIPOOPERFATURA,
                       PARAM.CDSERIENF,
                       PARAM.CDEMPRESA,
                       PARAM.CDINSCESTA,
                       PARAM.CDINSCMUNI,
                       PARAM.CDPRODFATURA,
                       PARAM.CDFILIALFATURA,
                       PARAM.NRPARCFATURA,
                       PARAM.QTPARCELASFATU,
                       PARAM.QTPARCELASFATUESC,
                       PARAM.QTDIASAPURACAO,
                       PARAM.NRDIACALCVENC,
                       PARAM.QTDIASFATUVENC,
                       PARAM.IDTPCALCVENC,
                       PARAM.IDMESCALCVENC,
                       PARAM.IDPREVCOBRFUT,
                       CODTRIB.CDTRIBPRODPADR,
                       PARAM.IDTIPONATUOPER,
                       PARAM.IDNATUOPER,
                       PARAM.IDMUNIPRESTSERV
                  FROM (
                    SELECT PFESC.NRTIPOORCAMENTO,
                           ITCONT.NRITEMOCONTRATO,
                           ITCONT.NRITEMCONTRATOPAI,
                           ITCONT.CDSERVORCA,
                           ITCONT.VRTOTITEMCONT,
                           ITCONT.NMITEMCONTRATO,
                           NVL(PFIT.IDTPFATURA, PFESC.IDTPFATURA) AS IDTPFATURA,
                           NVL(PFIT.CDTIPOOPERFATURA, PFESC.CDTIPOOPERFATURA) AS CDTIPOOPERFATURA,
                           NVL(PFIT.CDSERIENF, PFESC.CDSERIENF) AS CDSERIENF,
                           NVL(PFIT.CDEMPRESA, PFESC.CDEMPRESA) AS CDEMPRESA,
                           NVL(PFIT.CDINSCESTA, PFESC.CDINSCESTA) AS CDINSCESTA,
                           NVL(PFIT.CDINSCMUNI, PFESC.CDINSCMUNI) AS CDINSCMUNI,
                           NVL(ITCONT.CDPRODUTO, NVL(PFIT.CDPRODFATURA, NVL(PFESC.CDPRODFATURA, SERV.CDPRODFATSERV))) AS CDPRODFATURA,
                           NVL(CONT.CDFILIALORCA, NVL(PFIT.CDFILIALFATURA, PFESC.CDFILIALFATURA)) AS CDFILIALFATURA,
                           NVL(PFIT.NRPARCFATURA, NVL(PFESC.NRPARCFATURA, CONT.NRPARCNEGOCIO)) AS NRPARCFATURA,
                           NVL(PFIT.QTPARCELASFATU, PFESC.QTPARCELASFATU) AS QTPARCELASFATU,
                           NVL(PFESC.QTPARCELASFATU, 1) AS QTPARCELASFATUESC,
                           NVL(PFIT.QTDIASAPURACAO, PFESC.QTDIASAPURACAO) AS QTDIASAPURACAO,
                           NVL(PFIT.NRDIACALCVENC, PFESC.NRDIACALCVENC) AS NRDIACALCVENC,
                           NVL(PFIT.QTDIASFATUVENC, PFESC.QTDIASFATUVENC) AS QTDIASFATUVENC,
                           NVL(PFIT.IDTPCALCVENC, PFESC.IDTPCALCVENC) AS IDTPCALCVENC,
                           NVL(PFIT.IDMESCALCVENC, PFESC.IDMESCALCVENC) AS IDMESCALCVENC,
                           NVL(ITESC.IDPREVCOBRFUT, 'N') AS IDPREVCOBRFUT,
                           PFESC.IDTIPONATUOPER,
                           PFESC.IDNATUOPER,
                           NVL(CONT.IDMUNIPRESTSERV, NVL(PFESC.IDMUNIPRESTSERV, 'U')) AS IDMUNIPRESTSERV
                      FROM FCT_CAPAPARAMFATESC PFESC
                      LEFT JOIN FCT_ITESCOPOPROPOSTA ITESC
                        ON PFESC.NRTIPOORCAMENTO = ITESC.NRTIPOORCAMENTO
                       AND PFESC.NRORG           = ITESC.NRORG
                       AND ITESC.CDSERVORCA      = :CDSERVORCA
                      LEFT JOIN FCT_PARAMFATITESC PFIT
                        ON ITESC.NRTIPOORCAMENTO    = PFIT.NRTIPOORCAMENTO
                       AND ITESC.NRITESCOPOPROPOSTA = PFIT.NRITESCOPOPROPOSTA
                       AND ITESC.NRORG              = PFIT.NRORG
                     INNER JOIN FCT_CONTRATO CONT
                        ON PFESC.NRORG        = CONT.NRORG
                       AND CONT.NRSEQCONTRATO = :NRSEQCONTRATO
                     INNER JOIN FCT_ITEMCONTRATO ITCONT
                        ON CONT.NRSEQCONTRATO     = ITCONT.NRSEQCONTRATO
                       AND CONT.NRORG             = ITCONT.NRORG
                       AND ITCONT.NRITEMOCONTRATO = :NRITEMOCONTRATO
                      LEFT JOIN FCT_SERVICOORCA SERV
                        ON ITCONT.CDSERVORCA = SERV.CDSERVORCA
                       AND ITCONT.NRORG      = SERV.NRORG
                     WHERE PFESC.NRTIPOORCAMENTO = :NRTIPOORCAMENTO
                       AND PFESC.NRORG           = :NRORG
                  ) PARAM
                  LEFT JOIN CODTRIBPRODPADR CODTRIB
                    ON PARAM.CDEMPRESA    = CODTRIB.CDEMPRESA
                   AND PARAM.CDINSCMUNI   = CODTRIB.CDINSCMUNI
                   AND PARAM.CDPRODFATURA = CODTRIB.CDPRODUTO";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }

    public function excluirItemContrato(array $params) {
        $sql = "DELETE FROM FCT_ITEMCONTRATO
                WHERE NRITEMOCONTRATO = :NRITEMOCONTRATO
                  AND NRORG           = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     * Retorna o valor total de um item da negociação 
    **/
    public function getItensNegociacaoValoresContrato(array $params) {
        $sql = "SELECT VRTOTITEMCOMDESC, VRDESCITEM, CDSERVORCA
                FROM FCT_ITNEGOCIACAOCONTRATO 
                WHERE (NRITEMOCONTRATO = :NRITEMOCONTRATO OR :NRITEMOCONTRATO IS NULL)
                   AND NRNEGOCIACAOCONTRATO = :NRNEGOCIACAOCONTRATO
                   AND NRORG                = :NRORG";
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    /**
     * Busca o valor do serviço na tabela de preço
    **/
    public function getValorUnitarioTblPrecoItemContrato(array $params) {
        $sql = "SELECT DISTINCT DECODE(:CDTPMEDIDAVARIAVEL, NULL, TBLPREC.VRPRECO, VARPREC.VRPRECOVARIACAO) AS VRPRECO
                FROM FCT_ITPRECBASESERV TBLPREC
                LEFT JOIN FCT_VARIACAOPRECBASESERV VARPREC
                    ON TBLPREC.NRSEQITPRECBSERV = VARPREC.NRSEQITPRECBSERV
                    AND TBLPREC.NRORG           = VARPREC.NRORG
                WHERE TBLPREC.CDSERVORCA = :CDSERVORCA
                    AND TBLPREC.NRORG    = :NRORG
                    AND ((VARPREC.CDTPMEDIDAVARIAVEL = :CDTPMEDIDAVARIAVEL AND 
                        :QTMEDIDASERV BETWEEN QTMEDIDAINIVARIACAO AND QTMEDIDAFIMVARIACAO)
                        OR :CDTPMEDIDAVARIAVEL IS NULL)
                     AND TBLPREC.CDPRECBASESERV IN(SELECT DISTINCT CDPRECBASESERV
                                                   FROM FCT_TIPOORCAMENTO
                                                   WHERE NRTIPOORCAMENTO = :NRTIPOORCAMENTO)";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    /**
     * Atualiza o valor de um item na tabela de rateio
    **/
    public function atualizarValorRateioItens(array $params) {
        $sql = "UPDATE FCT_RELACESTRUTURACONT
                SET VRITEMRATEIO = ROUND(:VRTOTALITEM * (VRPERCRATEIO/100), 2),
                    CDOPERULTATU = :CDOPERULTATU,
                    DTULTATU     = SYSDATE
                WHERE NRITEMOCONTRATO    = :NRITEMOCONTRATO
                    AND NRSEQCONTRATO    = :NRSEQCONTRATO 
                    AND NRORG            = :NRORG
                    AND IDRELACESTRUTURA = 'RATEIO'";
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }

    public function excluirMedidaVariavelServico(array $params) {
        $sql = "DELETE FROM FCT_RELACMEDIDACONTRATO
                 WHERE NRSEQCONTRATO = :NRSEQCONTRATO
                   AND CDSERVORCA    = :CDSERVORCA
                   AND NRORG         = :NRORG";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }

    public function getMedidasVariaveisContratoSemItemRelacionado(array $params) {
        $sql = "SELECT NRMEDIDACONTRATO,
                       CDTPMEDIDAVARIAVEL,
                       QTMEDIDAVARIAVEL,
                       CDSERVORCA
                  FROM FCT_RELACMEDIDACONTRATO
                 WHERE NRSEQCONTRATO  = :NRSEQCONTRATO
                   AND NRORG          = :NRORG
                   AND IDELEMENTORAIZ IS NULL
                   AND CDSERVORCA     NOT IN (SELECT NVL(CDSERVORCA, 0)
                                                FROM FCT_ITEMCONTRATO
                                               WHERE NRSEQCONTRATO = :NRSEQCONTRATO
                                                 AND NRORG         = :NRORG)";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    public function atualizaMunicipioNegociacaoValores(array $params) {
        $sql = " UPDATE FCT_NEGOCIACAOCONTRATO 
                    SET CDPAISFAT      = :CDPAISFAT,
                        SGESTADOFAT    = :SGESTADOFAT,
                        CDMUNICIPIOFAT = :CDMUNICIPIOFAT,
                        DTULTATU       = SYSDATE,
                        CDOPERULTATU   = :OPERADOR,
                        NRORGULTATU    = :NRORG
                  WHERE NRNEGOCIACAOCONTRATO = :NRNEGOCIACAOCONTRATO
                    AND NRORG                = :NRORG";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }
    
}
?>